@@ -14,7 +14,6 @@ ITEM_METHODS = ['GET', 'PATCH', 'DELETE', 'PUT']
 ITEM_CACHE_CONTROL = ''
 ITEM_LOOKUP = True
 ITEM_LOOKUP_FIELD = ID_FIELD
-#ITEM_URL = '[a-f0-9]{24}'
 
 contacts = {
     'url': 'arbitraryurl',
@@ -70,6 +69,7 @@ contacts = {
         },
         'tid': {
             'type': 'objectid',
+            'nullable': True
         },
         'title': {
             'type': 'string',
@@ -86,6 +86,19 @@ contacts = {
         'id_list_fixed_len': {
             'type': 'list',
             'items': [{'type': 'objectid'}]
+        },
+        'dependency_field1': {
+            'type': 'string',
+            'default': 'default'
+        },
+        'dependency_field2': {
+            'type': 'string',
+            'dependencies': ['dependency_field1']
+        },
+        'read_only_field': {
+            'type': 'string',
+            'default': 'default',
+            'readonly': True
         }
     }
 }
@@ -102,8 +115,6 @@ users['item_title'] = 'user'
 users['additional_lookup']['field'] = 'username'
 
 invoices = {
-    #'item_lookup': False,
-    #'item_methods': ['GET'],
     'schema': {
         'inv_number': {'type': 'string'},
         'person': {
